@@ -156,7 +156,7 @@ def load_patch_image(dataset,imList):
     last_patch = 0
     for im_i in imList:
         num_patches = int(im_i.shape[0]/dataset.patch_size)*int(im_i.shape[1]/dataset.patch_size)
-        labels.append(np.reshape(np.array(range(last_patch,last_patch+num_patches)),(int(im_i.shape[0]/dataset.patch_size),int(im_i.shape[1]/dataset.patch_size))))
+        labels.append(np.transpose(np.reshape(np.array(range(last_patch,last_patch+num_patches)),(int(im_i.shape[1]/dataset.patch_size),int(im_i.shape[0]/dataset.patch_size)))))
         last_patch = last_patch+num_patches
 
     return np.concatenate(labels,1), labels
@@ -257,7 +257,7 @@ def topk_confident_patches(dataset,n_cell_types,
                             CropConfTissueComm,im,imList,PCL_reprsntions,count):
     '''
     '''
-    K = 25
+    K = int(np.ceil(5/dataset.processed_file_names*100))
 
     # Create label map. 
     labels,labels_list = load_patch_image(dataset,imList)
@@ -321,17 +321,6 @@ def calculate_pearson_correlation(matrix_0,matrix_1):
     cor_val = 0 if np.isnan(cor_val) else cor_val
     return cor_val
 
-def calculate_IoU_usingOtsu(matrix_0,matrix_1):
-    # Check if all values are equal and return an IoU of zeros
-    if matrix_0.max()==matrix_0.min() or matrix_1.max()==matrix_1.min():
-        return 0
-
-    matrix_0 = matrix_0>filters.threshold_otsu(matrix_0)
-    matrix_1 = matrix_1>filters.threshold_otsu(matrix_1)        
-    intersection = np.logical_and(matrix_0, matrix_1)
-    union = np.logical_or(matrix_0, matrix_1)
-    return np.sum(intersection) / np.sum(union)    
-
 def calculate_marker_colocalization(matrix,MarkerNames):
     # matrix: contains (number of patches, x_dimension, y_dimension, number of markers)
     Marker_Colocalization =  np.ones((len(MarkerNames),len(MarkerNames)))
@@ -350,6 +339,7 @@ def calculate_Cell_Size_circularity(filename, matrix):
     # selem = disk(2)
     Size = np.zeros((matrix.shape[3]))
     Eccentricity = np.zeros((matrix.shape[3]))
+    Granularity = np.zeros((matrix.shape[3]))
     for mk in range(matrix.shape[3]):
 
         # Calculate global threshold
@@ -361,19 +351,21 @@ def calculate_Cell_Size_circularity(filename, matrix):
         for patch in range(matrix.shape[0]):
             
             if matrix[patch,:,:].sum()>0 and (matrix[patch,:,:].max()-matrix[patch,:,:].min())>0:
-                logical = matrix[patch,:,:]>thr
-                Size_eccentricity[0] += logical.sum() # Area
-                Size_eccentricity[1] += min((4*3.14159*logical.sum())/(perimeter(logical,neighbourhood=4)**2),1) # Eccentricity
+                logical = matrix[patch,:,:,mk]>thr
+                output = cv2.connectedComponentsWithStats(np.array(logical,dtype=np.uint8), 4, cv2.CV_32S)
 
-                count += 1
+                if output[0]>1: # at least one component
+                    Size[mk] += output[2][1:,4].mean()                    
+                    Eccentricity[mk] += min((4*3.14159*(output[1]==1).sum())/(perimeter((output[1]==1),neighbourhood=4)**2),1) # Eccentricity
+                    Granularity[mk] += output[0]-1
+
+                    count += 1
         
         Size[mk] /= count
         Eccentricity[mk] /= count
+        Granularity[mk] /= count
 
-                # if patch<4:
-                #     cv2.imwrite(filename+"Patch_"+str(patch)+"_Size_"+str(logical.sum())+"_Ecc_"+str((4*3.14159*logical.sum())/(perimeter(logical,neighbourhood=4)**2))+"_Raw.png", matrix[patch,:,:]/(matrix[patch,:,:].max())*255)
-                #     cv2.imwrite(filename+"Patch_"+str(patch)+"_Size_"+str(logical.sum())+"_Ecc_"+str((4*3.14159*logical.sum())/(perimeter(logical,neighbourhood=4)**2))+"_Logical.png", logical*255)
-    return Size_eccentricity/matrix.shape[0]
+    return Size, Eccentricity, Granularity
 
 def extract_topk_patches_from_cohort(dataset, CropConf, Marker_Names,cell_type,Thresholds):
     '''
@@ -393,9 +385,14 @@ def extract_topk_patches_from_cohort(dataset, CropConf, Marker_Names,cell_type,T
     k=100
     topkPatches=[]
     # Create a heatmap marker using topk patches
-    heatmapMarkerExpression = np.zeros((7,len(CropConf),len(Marker_Names))) # Number of TMEs x number of markers
+    heatmapMarkerExpression = np.zeros((len(CropConf),len(Marker_Names))) # Number of TMEs x number of markers
     heatmap_Colocalization = np.zeros((len(CropConf),len(Marker_Names),len(Marker_Names))) # Number of TMEs x Number of Markers x Number of Markers
-    heatmap_CellSize_circularity = np.zeros((len(CropConf),2)) # size and circularity of markers
+    heatmap_Size = np.zeros((len(CropConf),len(Marker_Names))) # size and circularity of markers
+    heatmap_Eccentricity = np.zeros((len(CropConf),len(Marker_Names))) # size and circularity of markers
+    heatmap_Granularity = np.zeros((len(CropConf),len(Marker_Names))) # size and circularity of markers
+
+    # Make directory
+    makedirs(thisfolder+'Prototypes/')
 
     # Use CropCOnf, that saves a lot of patches...
     for n_cell_type ,CropConf_i in enumerate(CropConf):
@@ -407,23 +404,13 @@ def extract_topk_patches_from_cohort(dataset, CropConf, Marker_Names,cell_type,T
         topkPheno = np.array([CCP[1] for CCP in CropConf_i]).argsort()[-k:]               
         
         heatmap_Colocalization[n_cell_type,:,:] = calculate_marker_colocalization(np.array([CropConf_i[t][0] for t in topkPheno]),Marker_Names)    
-        heatmap_CellSize_circularity[n_cell_type,:] = calculate_Cell_Size_circularity(thisfolder,np.array([CropConf_i[t][0] for t in topkPheno]))
+        heatmap_Size[n_cell_type,:], heatmap_Eccentricity[n_cell_type,:], heatmap_Granularity[n_cell_type,:] = calculate_Cell_Size_circularity(thisfolder,np.array([CropConf_i[t][0] for t in topkPheno]))
 
         # Save topkPheno to heatMarkerMap. Mean
         MarkerExpression = np.array([CropConf_i[t][0].mean((0,1)) for t in topkPheno])
-        
-        # # Save marker expression of patches individually
-        # if cell_type=='Phenotypes':        
-        #     save_heatmap_with_names(thisfolder+'TME_{}_patchExpression.png'.format(n_cell_type+1),MarkerExpression.mean((1,2)),Marker_Names)            
-        # else:
-        #     expr = MarkerExpression[:,::int(dataset.patch_size/10),::int(dataset.patch_size/10),:]            
-        #     save_heatmap_with_names(thisfolder+'TME_{}_patchExpression.png'.format(n_cell_type+1),np.array([expr[:,:,:,i].flatten() for i in range(expr.shape[3])]).T,Marker_Names)
-
-        # MarkerExpression = np.reshape(MarkerExpression,(MarkerExpression.shape[0]*MarkerExpression.shape[1]*MarkerExpression.shape[2],MarkerExpression.shape[3]))
-        heatmapMarkerExpression[0,n_cell_type,:] = np.mean(MarkerExpression,axis=0)                    
-        # for n_i, i in enumerate(Thresholds[1:]):
-        #     heatmapMarkerExpression[n_i+1,n_cell_type,:] = np.percentile(MarkerExpression,i,axis=0)                    
-
+              
+        heatmapMarkerExpression[n_cell_type,:] = np.mean(MarkerExpression,axis=0)                    
+      
         # Save Image in RGB
         ImwithKPatches = np.zeros((dataset.patch_size*mult_1*int(np.sqrt(k))+mult_2*int(np.sqrt(k)),dataset.patch_size*mult_1*int(np.sqrt(k))+mult_2*int(np.sqrt(k)),CropConf_i[0][0].shape[2]))
         for t_n, t in enumerate(topkPheno):
@@ -433,30 +420,29 @@ def extract_topk_patches_from_cohort(dataset, CropConf, Marker_Names,cell_type,T
             # Assign patch to Image
             ImwithKPatches[int(row*mult_1*dataset.patch_size+row*mult_2):int((row+1)*mult_1*dataset.patch_size+row*mult_2),int(col*mult_1*dataset.patch_size+col*mult_2):int((col+1)*mult_1*dataset.patch_size+col*mult_2),:] = CropConf_i[t][0]
         
-        if len(Marker_Names)<10:
-            # Fill unassigned patches with zeroes.
-            for t_n in range(len(topkPheno),k):
-                row = np.floor(t_n/int(k**0.5))
-                col = np.mod(t_n,int(k**0.5))
-                # Assign patch to Image
-                ImwithKPatches[int(row*mult_1*dataset.patch_size+row*mult_2):int((row+1)*mult_1*dataset.patch_size+row*mult_2),int(col*mult_1*dataset.patch_size+col*mult_2):int((col+1)*mult_1*dataset.patch_size+col*mult_2),:] = 0.0
-                            
-            # RGBImwithKPatches = dataset.nPlex2RGB(ImwithKPatches)
-            imwrite(thisfolder+'Cell_type_{}_Raw.tiff'.format(n_cell_type+1),np.moveaxis(ImwithKPatches,2,0), photometric='minisblack')
-            
-            # Save Certainty of this Phenotype
-            plt.close('all')
-            plt.figure()
-            n, bins, patches = plt.hist(np.array([i[1] for i in CropConf_i]), 100, color=cm.jet_r(int(n_cell_type*(255/int(len(CropConf))))), alpha=1)            
-            plt.ylabel('Number of Patches',fontsize=16)            
-            plt.xlabel('Level of TME certainty',fontsize=16)
-            plt.title('Histogram of TME ' + str(n_cell_type+1) + ' certainty',fontsize=16)
-            plt.savefig(thisfolder+'ConfidenceHistogram_{}.png'.format(n_cell_type+1), format="PNG",dpi=600)                                         
+        # Fill unassigned patches with zeroes.
+        for t_n in range(len(topkPheno),k):
+            row = np.floor(t_n/int(k**0.5))
+            col = np.mod(t_n,int(k**0.5))
+            # Assign patch to Image
+            ImwithKPatches[int(row*mult_1*dataset.patch_size+row*mult_2):int((row+1)*mult_1*dataset.patch_size+row*mult_2),int(col*mult_1*dataset.patch_size+col*mult_2):int((col+1)*mult_1*dataset.patch_size+col*mult_2),:] = 0.0
+                        
+        # RGBImwithKPatches = dataset.nPlex2RGB(ImwithKPatches)
+        imwrite(thisfolder+'Prototypes/Cell_type_{}_Raw.tiff'.format(n_cell_type+1),np.moveaxis(ImwithKPatches,2,0), photometric='minisblack')
+        
+        # Save Certainty of this Phenotype
+        plt.close('all')
+        plt.figure()
+        n, bins, patches = plt.hist(np.array([i[1] for i in CropConf_i]),range=(0, 1), color=cm.jet_r(int(n_cell_type*(255/int(len(CropConf))))), alpha=1)            
+        plt.ylabel('Number of Patches',fontsize=16)            
+        plt.xlabel('Level of TME certainty',fontsize=16)
+        plt.title('Histogram of TME ' + str(n_cell_type+1) + ' certainty',fontsize=16)
+        plt.savefig(thisfolder+'Prototypes/ConfidenceHistogram_{}.png'.format(n_cell_type+1), format="PNG",dpi=600)                                         
 
         # Assign topkPheno Patches
         topkPatches+=[CropConf_i[t] for t in topkPheno]     
 
-    return heatmapMarkerExpression, heatmap_Colocalization, heatmap_CellSize_circularity
+    return heatmapMarkerExpression, heatmap_Colocalization, heatmap_Size, heatmap_Eccentricity, heatmap_Granularity
 
 def save_heatmap_raw_and_normalized(filename, heatmap, TME_names,Colormap,Marker_Names):    
     y_ticklabels = TME_names[heatmap.sum(1)!=0]
@@ -469,6 +455,11 @@ def save_heatmap_raw_and_normalized(filename, heatmap, TME_names,Colormap,Marker
     h_E_Fig = sns.clustermap(heatmap[heatmap.sum(1)!=0,:],col_cluster=cluster, row_cluster=cluster, row_colors=c_map,xticklabels=Marker_Names,yticklabels=y_ticklabels, linewidths=0.5, cmap="bwr")            
     h_E_Fig.savefig(filename+'_Raw.png',dpi=600) 
 
+    # Save Excel.
+    df = pd.DataFrame(heatmap[heatmap.sum(1)!=0,:], columns = Marker_Names)
+    df.index = y_ticklabels
+    df.to_excel(filename+'_Raw.xlsx')  
+
     # Figure Heatmap Min is 0 and max is 1 Values    
     h_E_COL_MinMax = heatmap[heatmap.sum(1)!=0,:] - heatmap[heatmap.sum(1)!=0,:].min(0,keepdims=True)
     h_E_COL_MinMax = h_E_COL_MinMax/h_E_COL_MinMax.max(0,keepdims=True)
@@ -511,7 +502,7 @@ def save_heatmap_raw_and_normalized(filename, heatmap, TME_names,Colormap,Marker
         h_E_Fig = sns.clustermap(h_E_COL_Norm,col_cluster=True, vmin=-2, vmax=2, row_cluster=True, row_colors=c_map,xticklabels=Marker_Names,yticklabels=y_ticklabels, linewidths=0.5, cmap="bwr")            
         h_E_Fig.savefig(filename+'_Norm_rows.png',dpi=600) 
 
-def save_heatMapMarker_and_barplot(dataset, heatmapMarkerExpression, heatmapMarkerColocalization, heatmap_CellSize_circularity, CropConf,Marker_Names,cell_type,Thresholds):
+def save_heatMapMarker_and_barplot(dataset, heatmapMarkerExpression, heatmap_Colocalization, heatmap_Size, heatmap_Eccentricity, heatmap_Granularity, CropConf,Marker_Names,cell_type,Thresholds):
     '''
     '''
     if cell_type=='Phenotypes':
@@ -519,32 +510,41 @@ def save_heatMapMarker_and_barplot(dataset, heatmapMarkerExpression, heatmapMark
     else:
         abrev = 'N'
 
-    Colormap=cm.jet(range(0,255,int(255/heatmapMarkerExpression.shape[1])))[:,:3]
+    Colormap=cm.jet(range(0,255,int(255/heatmapMarkerExpression.shape[0])))[:,:3]
 
-    # Save heatmapmarker to disk
-    for n_j, j in enumerate(Thresholds):
-        save_heatmap_raw_and_normalized(dataset.bioInsights_dir_cell_types+cell_type+'/heatmap_MarkerExpression_'+str(j), 
-                                        heatmapMarkerExpression[n_j,:,:], np.array([abrev+str(i+1) for i in range(len(CropConf))]),
+    # Save heatmapmarker to disk    
+    makedirs(dataset.bioInsights_dir_cell_types+cell_type+'/Marker_Expression/')
+    save_heatmap_raw_and_normalized(dataset.bioInsights_dir_cell_types+cell_type+'/Marker_Expression/heatmap', 
+                                        heatmapMarkerExpression, np.array([abrev+str(i+1) for i in range(len(CropConf))]),
                                         Colormap,Marker_Names)
-        # save_heatmap_raw_and_normalized(dataset.bioInsights_dir_cell_types+cell_type+'/heatmap_MarkerExpression_Colocalization_'+str(j), 
-        #                                 np.concatenate((heatmapMarkerExpression[n_j,:,:],heatmapMarkerColocalization),axis=1), np.array([abrev+str(i+1) for i in range(len(CropConf))]),
-        #                                 Colormap,Marker_Names+['_'.join(i) for i in itertools.combinations(Marker_Names,2)])
-        
 
-    # try:
-    #     save_heatmap_raw_and_normalized(dataset.bioInsights_dir_cell_types + cell_type + '/heatmap_Size_Circularity', heatmap_CellSize_circularity, np.array([abrev+str(i+1) for i in range(len(CropConf))]),Colormap,['Nucleus Size', 'Nucleus circularity'])
-    # except:
-    #     print('')
-    # save_heatmap_raw_and_normalized(dataset.bioInsights_dir_cell_types + cell_type + '/heatmap_Colocalization', heatmapMarkerColocalization, np.array([abrev+str(i+1) for i in range(len(CropConf))]),Colormap,['_'.join(i) for i in itertools.combinations(Marker_Names,2)])
+    makedirs(dataset.bioInsights_dir_cell_types+cell_type+'/Morphology/')
+    save_heatmap_raw_and_normalized(dataset.bioInsights_dir_cell_types+cell_type+'/Morphology/Heatmap_Size', 
+                                        heatmap_Size, np.array([abrev+str(i+1) for i in range(len(CropConf))]),
+                                        Colormap,Marker_Names)            
 
-    # Calculate Phenotype abundance across all patients
-    plt.close('all')
-    plt.figure()
-    sns.set(font_scale=1.0)
-    BarPlotPresenceOfPhenotypes = sns.barplot(x=[abrev+str(i+1) for i in range(len(CropConf))] ,y=np.array([len(i) for i in CropConf]),palette=Colormap)      
-    BarPlotPresenceOfPhenotypes.set(xlabel=cell_type, ylabel = 'Number of patches',title='Histogram of abundance across the patient cohort')                
-    BarPlotPresenceOfPhenotypes.set_xticklabels([abrev+str(i+1) for i in range(len(CropConf))], size=7)
-    plt.savefig(dataset.bioInsights_dir_cell_types + cell_type +'/Barplot_cell_types.png',dpi=600)
+    save_heatmap_raw_and_normalized(dataset.bioInsights_dir_cell_types+cell_type+'/Morphology/heatmap_Eccentricity', 
+                                        heatmap_Eccentricity, np.array([abrev+str(i+1) for i in range(len(CropConf))]),
+                                        Colormap,Marker_Names)         
+
+    save_heatmap_raw_and_normalized(dataset.bioInsights_dir_cell_types+cell_type+'/Morphology/heatmap_Granularity', 
+                                        heatmap_Eccentricity, np.array([abrev+str(i+1) for i in range(len(CropConf))]),
+                                        Colormap,Marker_Names)     
+
+    # Correlation matrix
+    makedirs(dataset.bioInsights_dir_cell_types+cell_type+'/Marker_Colocalization/')
+    mask = np.triu(np.ones_like(heatmap_Colocalization[0,:,:], dtype=np.bool))
+    for TME in range(len(heatmap_Colocalization)):
+        plt.close('all')
+        plt.figure()
+        heatmap = sns.heatmap(heatmap_Colocalization[TME,:,:], mask=mask, vmin=heatmap_Colocalization.min(), vmax=heatmap_Colocalization.max(),xticklabels=Marker_Names,yticklabels=Marker_Names, annot=True, cmap='BrBG')
+        heamap = heatmap.get_figure()
+        heamap.savefig(dataset.bioInsights_dir_cell_types+cell_type+'/Marker_Colocalization/heatmap_Colocalization_'+abrev+str(TME+1)+'.png',dpi=600) 
+        df = pd.DataFrame(heatmap_Colocalization[TME,:,:], columns = Marker_Names)
+        df.index = Marker_Names
+        df.to_excel(dataset.bioInsights_dir_cell_types+cell_type+'/Marker_Colocalization/heatmap_Colocalization_'+abrev+str(TME+1)+'.xlsx')  
+
+       
 
 def neigh_comp(TC,phenoInd):
     InteractivityVect = np.zeros(len(phenoInd))
