@@ -998,15 +998,15 @@ class NaroNet_dataset(torch.utils.data.Dataset):
         Channels, Marker_Names = utilz.load_channels(self.root+'Raw_Data/')
 
         # Extract topk patches of phenotypes
-        heatmapMarkerExpression, heatmap_MarkerColocalization, heatmap_CellSize_circularity = extract_topk_patches_from_cohort(self, CropConfPheno, [Marker_Names[c] for c in Channels],'Phenotypes',Thresholds)
-        save_heatMapMarker_and_barplot(self, heatmapMarkerExpression, heatmap_MarkerColocalization, heatmap_CellSize_circularity,CropConfPheno,[Marker_Names[c] for c in Channels],'Phenotypes',Thresholds)
+        heatmapMarkerExpression, heatmap_Colocalization, heatmap_Size, heatmap_Eccentricity, heatmap_Granularity = extract_topk_patches_from_cohort(self, CropConfPheno, [Marker_Names[c] for c in Channels],'Phenotypes',Thresholds)
+        save_heatMapMarker_and_barplot(self, heatmapMarkerExpression, heatmap_Colocalization, heatmap_Size, heatmap_Eccentricity, heatmap_Granularity,CropConfPheno,[Marker_Names[c] for c in Channels],'Phenotypes',Thresholds)
 
         # Extract topk patches of neighborhoods
-        heatmapMarkerExpression, heatmap_MarkerColocalization, heatmap_CellSize_circularity = extract_topk_patches_from_cohort(self, CropConfTissueComm, [Marker_Names[c] for c in Channels],'Neighborhoods',Thresholds)
-        save_heatMapMarker_and_barplot(self, heatmapMarkerExpression, heatmap_MarkerColocalization, heatmap_CellSize_circularity,CropConfTissueComm,[Marker_Names[c] for c in Channels],'Neighborhoods',Thresholds)
+        heatmapMarkerExpression, heatmap_Colocalization, heatmap_Size, heatmap_Eccentricity, heatmap_Granularity  = extract_topk_patches_from_cohort(self, CropConfTissueComm, [Marker_Names[c] for c in Channels],'Neighborhoods',Thresholds)
+        save_heatMapMarker_and_barplot(self, heatmapMarkerExpression, heatmap_Colocalization, heatmap_Size, heatmap_Eccentricity, heatmap_Granularity,CropConfTissueComm,[Marker_Names[c] for c in Channels],'Neighborhoods',Thresholds)
         
         # Neighborhoods to Phenotypes
-        obtain_neighborhood_composition(self,CropConfPheno,CropConfTissueComm)
+        # obtain_neighborhood_composition(self,CropConfPheno,CropConfTissueComm)
 
     def visualize_results(self, clusters, IndexAndClass):        
         '''
@@ -1014,39 +1014,20 @@ class NaroNet_dataset(torch.utils.data.Dataset):
         '''
 
         # Obtain Neigh-to-Pheno, and Area-to-Neigh heatmap.
-        # for ClusteringTrheshold in [0,50,75,90,99]:
-        #     Area_to_Neighborhood_to_Phenotype(self,clusters,IndexAndClass,ClusteringTrheshold)     
+        for ClusteringTrheshold in [0,50,75,90,99]:
+            Area_to_Neighborhood_to_Phenotype(self,clusters,IndexAndClass,ClusteringTrheshold)     
 
         # Obtain cell-types examples
         self.ObtainPhenoExamples(IndexAndClass,clusters,[25])                
         
-        # # Cell-types abundances
-        # heatmapClusterPresence, labels = self.clusterPresence(clusters, IndexAndClass,[50], isTraining=False)                        
+        # Cell-types abundances
+        heatmapClusterPresence, labels = self.clusterPresence(clusters, IndexAndClass,[50], isTraining=False)                        
         
-        # # Differential abundance analysis
-        # _, _, _, _, _ = differential_abundance_analysis(self, heatmapClusterPresence, labels, IndexAndClass,False)
+        # Differential abundance analysis
+        _, _, _, _, _ = differential_abundance_analysis(self, heatmapClusterPresence, labels, IndexAndClass,False)
 
         # Apply clusters and attention to images
-        All_TMEs_in_Image(self, clusters, IndexAndClass)        
-   
-    def obtain_intersec_acc(self,model, clusters, IndexAndClass, num_classes,mean_STD, args,TrainingClusterMapEpoch):        
-
-        isTraining = True
-                   
-        # Obtain patient-CLusterMap and calculate statisticalSignificance between groups.       
-        self.TrainingClusterMapEpoch =  TrainingClusterMapEpoch
-        heatmapClusterPresence, labels = self.clusterPresence(clusters, IndexAndClass, num_classes, 0, isTraining=True)  
-        statisticalTest, unrestrictedLoss, Top1PerPatient, patient_Ineach_subgroup, real_class_confidence = differential_abundance_analysis(self, heatmapClusterPresence, labels, IndexAndClass,True)
-
-        # Obtain Segmentation index for Statistical Value.
-        if ('Synthetic' in self.root):
-            IntersectionIndex = ObtainMultivariateIntersectInSynthetic(self, Top1PerPatient, clusters, IndexAndClass, num_classes, args['isAttentionLayer'],True)
-        else:
-            IndexAndClass_Best_and_Worst = Best_and_Worst(statisticalTest,num_classes,unrestrictedLoss)
-            IntersectionIndex = self.ObtainMultivariateIntersectInReal(statisticalTest, clusters, self.IndexAndClass_onePerClass, num_classes, args['isAttentionLayer'],0.5)
-            IntersectionIndex = self.ObtainMultivariateIntersectInReal(statisticalTest, clusters, self.IndexAndClass_onePerClass, num_classes, args['isAttentionLayer'],0.75)
-            IntersectionIndex = self.ObtainMultivariateIntersectInReal(statisticalTest, clusters, self.IndexAndClass_onePerClass, num_classes, args['isAttentionLayer'],0.95)
-        return np.array(IntersectionIndex[1]).mean()
+        All_TMEs_in_Image(self, clusters, IndexAndClass[:int(self.args['Pheno_Perc_Pat']*len(IndexAndClass))])           
 
 def get_BioInsights(path, parameters):
     '''
