@@ -377,7 +377,7 @@ def parameters(path, debug):
 
         # General
         args['training_MODE'] = 'CrossValidation' # TrainALL, NestedCrossValidation, CrossValidation # Whether to use nested cross validation or cross validation.        
-        args['folds'] = 10
+        args['folds'] = 12
         args['device'] = 'cuda:0'
         args['Batch_Normalization'] = True
         args['dataAugmentationPerc'] = 0 #0 if debug=='Index' else hp.choice("dataAugmentationPerc", [0,0.0001,0.001]) if debug=='Object' else 0  
@@ -675,7 +675,7 @@ def parameters(path, debug):
         # Bioinsights parameters
         args['Bio_save_Orig_im'] = True
         args['TSNE_Perc_Pat'] = 0 # Percentage of patients used to calculate TSNE. 1 is 100% of patients
-        args['Pheno_Perc_Pat'] = 0.5 # Percentage of patients used to calculate TSNE. 1 is 100% of patients
+        args['Pheno_Perc_Pat'] = 1 # Percentage of patients used to calculate TSNE. 1 is 100% of patients
 
     else:
         
